<?php

/*
|--------------------------------------------------------------------------
| Application Routes
|--------------------------------------------------------------------------
|
| Here is where you can register all of the routes for an application.
| It is a breeze. Simply tell Lumen the URIs it should respond to
| and give it the Closure to call when that URI is requested.
|
*/
use App\Order;
use App\Menu;
use App\Food;
use App\Ingredient;
use Illuminate\Http\Request;

function jsonp($response, $request){
    return response()->json($response)->setCallback($request->input('callback'));
}

$app->routeMiddleware(['roleAuth' => App\Http\Middleware\RoleAuthenticate::class]);

$app->get('/api/config', function (Request $request) {
    $json = json_decode(file_get_contents(dirname(__FILE__) . '/../../public/config.json'));
    return jsonp($json, $request);
});

$app->get('/api/getAll', function (Request $request) {
    $menus = Menu::with('foods')->get();
    $foods = Food::with('ingredients')->get();
    $ingredients = Ingredient::all();
    $response = ['menus' => $menus, 'foods' => $foods, 'ingredients' => $ingredients];
    return jsonp($response, $request);
});

// MENU
$app->get('/api/menu/create', ['middleware' => 'roleAuth:boss', function (Request $request) {
    $menu = Menu::create(['name' => $request->name, 'price' => $request->price]);
    for ($i = 0; $request->has('foods_' . $i . '_id'); $i++){
      $food = Food::find($request->input('foods_' . $i . '_id'));
      $menu->foods()->attach($food->id);
    }
    $menu->save();
    return jsonp(['error' => 0], $request);
}]);
$app->get('/api/menu/get/{id}', function (Request $request, $id) {
    $menu = Menu::with('foods')->find($id);
    return jsonp($menu, $request);
});
$app->get('/api/menu/update/{id}', ['middleware' => 'roleAuth:boss', function (Request $request, $id) {
    $menu = Menu::find($id);
    $menu->name = $request->name;
    $menu->price = $request->price;
    $menu->foods()->detach();
    for ($i = 0; $request->has('foods_' . $i . '_id'); $i++){
      $food = Food::find($request->input('foods_' . $i . '_id'));
      $menu->foods()->attach($food->id);
    }
    $menu->save();
    return jsonp(['error' => 0], $request);
}]);
$app->get('/api/menu/delete/{id}', ['middleware' => 'roleAuth:boss', function (Request $request, $id) {
    return Menu::destroy($id);
}]);
// FOOD
$app->get('/api/food/create', ['middleware' => 'roleAuth:boss', function (Request $request) {
    $food = Food::create(['name' => $request->name, 'price' => $request->input('price', null)]);
    for ($i = 0; $request->has('requiredIngredients_' . $i . '_id'); $i++){
      $ingredient = Ingredient::find($request->input('requiredIngredients_' . $i . '_id'));
      $food->ingredients()->attach($ingredient->id);
    }
    for ($i = 0; $request->has('optionalIngredients_' . $i . '_id'); $i++){
      $ingredient = Ingredient::find($request->input('optionalIngredients_' . $i . '_id'));
      $food->ingredients()->attach($ingredient->id, ['required' => false]);
    }
    $food->save();
    return jsonp(['error' => 0], $request);
}]);
$app->get('/api/food/get/{id}', function (Request $request, $id) {
    $food = Food::with('ingredients')->find($id);
    return jsonp($food, $request);
});
$app->get('/api/food/update/{id}', ['middleware' => 'roleAuth:boss', function (Request $request, $id) {
    $food = Food::find($id);
    $food->name = $request->name;
    $food->price = $request->input('price', null);
    $food->ingredients()->detach();
    for ($i = 0; $request->has('requiredIngredients_' . $i . '_id'); $i++){
      $ingredient = Ingredient::find($request->input('requiredIngredients_' . $i . '_id'));
      $food->ingredients()->attach($ingredient->id);
    }
    for ($i = 0; $request->has('optionalIngredients_' . $i . '_id'); $i++){
      $ingredient = Ingredient::find($request->input('optionalIngredients_' . $i . '_id'));
      $food->ingredients()->attach($ingredient->id, ['required' => false]);
    }
    $food->save();
    return jsonp(['error' => 0], $request);
}]);
$app->get('/api/food/delete/{id}', ['middleware' => 'roleAuth:boss', function (Request $request, $id) {
    Food::destroy($id);
}]);
// INGREDIENT
$app->get('/api/ingredient/create', ['middleware' => 'roleAuth:boss', function (Request $request) {
    Ingredient::create(['name' => $request->name]);
}]);
$app->get('/api/ingredient/get/{id}', function (Request $request, $id) {
    $ingredient = Ingredient::find($id);
    return jsonp($ingredient, $request);
});
$app->get('/api/ingredient/update/{id}', ['middleware' => 'roleAuth:boss', function (Request $request, $id) {
    $ingredient = Ingredient::find($id);
    $ingredient->name = $request->name;
    $ingredient->save();
}]);
$app->get('/api/ingredient/delete/{id}', ['middleware' => 'roleAuth:boss', function (Request $request, $id) {
    Ingredient::destroy($id);
}]);
// ORDER
$app->get('/api/order/create', ['middleware' => 'roleAuth:boss,waiter', function (Request $request) {
    $order = new Order;
    $order->description = $request->description;
    $order->save();
    for ($i = 0; $request->has("foods_${i}_id"); $i++){
      $food_order_id = DB::table('food_order')->insertGetId(
        [
          'food_id' => $request->input("foods_${i}_id"),
          'order_id' => $order->id,
          'quantity' => $request->input("foods_${i}_quantity")
        ]
      );
      for ($j = 0; $request->has("foods_${i}_lessIng_${j}_id"); $j++){
        DB::table('lessIng_food')->insert(
        [
          'food_order_id' => $food_order_id,
          'ingredient_id' => $request->input("foods_${i}_lessIng_${j}_id")
        ]
        );
      }
    }
    for ($i = 0; $request->has("menus_${i}_id"); $i++){
      $menu_order_id = DB::table('menu_order')->insertGetId(
        [
          'menu_id' => $request->input("menus_${i}_id"),
          'order_id' => $order->id,
          'quantity' => $request->input("menus_${i}_quantity")
        ]
      );
      for ($j = 0; $request->has("menus_${i}_foods_${j}_id"); $j++){
        for ($k = 0; $request->has("menus_${i}_foods_${j}_lessIng_${k}_id"); $k++){
          DB::table('lessIng_food_menu')->insert(
          [
            'menu_order_id' => $menu_order_id,
            'food_id' => $request->input("menus_${i}_foods_${j}_id"),
            'ingredient_id' => $request->input("menus_${i}_foods_${j}_lessIng_${k}_id")
          ]
          );
        }
      }
    }
    return jsonp(['error' => 0], $request);
}]);

$app->get('/api/order/getAll', ['middleware' => 'roleAuth:boss,waiter,chef', function (Request $request) {
    $orders = Order::all();
    return jsonp($orders, $request);
}]);

$app->get('/api/order/get/{id}', ['middleware' => 'roleAuth:boss,waiter,chef', function (Request $request, $id) {
    $order = Order::find($id);
    return jsonp($order, $request);
}]);

$app->get('/api/order/delete/{id}', ['middleware' => 'roleAuth:boss,waiter,chef', function (Request $request, $id) {
    Order::destroy($id);
    return jsonp(['error' => 0], $request);
}]);
